///////////////////////////////////////////////////////////////////////////////
// NABLA - a Numerical Analysis Based LAnguage                               //
//                                                                           //
// Copyright (C) 2014~2017 CEA/DAM/DIF                                       //
// IDDN.FR.001.520002.000.S.P.2014.000.10500                                 //
//                                                                           //
// Contributor(s): CAMIER Jean-Sylvain - Jean-Sylvain.Camier@cea.fr          //
//                                                                           //
// This software is a computer program whose purpose is to translate         //
// numerical-analysis specific sources and to generate optimized code        //
// for different targets and architectures.                                  //
//                                                                           //
// This software is governed by the CeCILL license under French law and      //
// abiding by the rules of distribution of free software. You can  use,      //
// modify and/or redistribute the software under the terms of the CeCILL     //
// license as circulated by CEA, CNRS and INRIA at the following URL:        //
// "http://www.cecill.info".                                                 //
//                                                                           //
// The CeCILL is a free software license, explicitly compatible with         //
// the GNU GPL.                                                              //
//                                                                           //
// As a counterpart to the access to the source code and rights to copy,     //
// modify and redistribute granted by the license, users are provided only   //
// with a limited warranty and the software's author, the holder of the      //
// economic rights, and the successive licensors have only limited liability.//
//                                                                           //
// In this respect, the user's attention is drawn to the risks associated    //
// with loading, using, modifying and/or developing or reproducing the       //
// software by the user in light of its specific status of free software,    //
// that may mean that it is complicated to manipulate, and that also         //
// therefore means that it is reserved for developers and experienced        //
// professionals having in-depth computer knowledge. Users are therefore     //
// encouraged to load and test the software's suitability as regards their   //
// requirements in conditions enabling the security of their systems and/or  //
// data to be ensured and, more generally, to use and operate it in the      //
// same conditions as regards security.                                      //
//                                                                           //
// The fact that you are presently reading this means that you have had      //
// knowledge of the CeCILL license and that you accept its terms.            //
//                                                                           //
// See the LICENSE file for details.                                         //
///////////////////////////////////////////////////////////////////////////////

// ****************************************************************************
// αβγδεζηθικλμνξοπρςστυφχψωΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩ
// ∀ ℝℕℤ ²³√½⅓¼⅛
// ****************************************************************************

const ℝ π=3.1415926535897932384626433832795028841971693993751;

const ℕ binMax=100;
// Devraient migrer en global pour être capable de placer les atomics
ℝ histo[binMax];
ℝ Θbinning[binMax];

// Dump de l'histogramme dans ce fichier
ofstream histoData("/tmp/histo.data");


// ****************************************************************************
// * Options
// ****************************************************************************
options{
  // Mesh options
  ℝ LENGTH                    = 1.0;
  ℕ X_EDGE_ELEMS              = 8;
  ℕ Y_EDGE_ELEMS              = 8;
  ℕ Z_EDGE_ELEMS              = 1;
  // Paramètres d'entrée
  ℝ option_νδt = 0.05; // Fréquence de collision x pas de temps
  ℾ option_particle_frame = true;
  // Numerical options
  ℝ option_ε_frame_change = 1.0e-15;
  ℝ option_Γα = 1.0+0.5;
  ℝ option_Γβ = 2.0;
  // Yet required options for backend
  ℝ option_δt_initial = 0.1;
  ℝ option_stoptime = 1.0;
  ℕ option_max_iterations = 32768;   // ! utilisé en dur dans les backends
};


// **************************************************************************
// * Forward declaration
// ****************************************************************************
void iniRandom(double,double);
double gamma_rand(void);


// **************************************************************************
// * Particles Variables
// ****************************************************************************
particles{
  // Repère du laboratoire
  ℝ αLab,βLab,γLab; // Cosinus directeurs
  // Repère de la particule
  ℝ α,β,γ;
};


// **************************************************************************
// * Globals
// ****************************************************************************
//global{ ℝ seed;};


// ****************************************************************************
// * Initialization Part @ ]-∞,-0.0[
// ****************************************************************************
void iniGlobals(void) @ -5.0{
  //δt=option_δt_initial;
  printf("\n[7m[iniGlobals] δt=%f[m", (double)δt);
  assert(δt>=0.0);
}

void iniRandom(void) @ -5.0 {
  info()<<"[7;37m[iniRandom][m";
  iniRandom(option_Γα,option_Γβ);
  //info()<<"\n"<<gamma_rand();
}

// ****************************************************************************
// * initialization α, β et γ dans le repère du labo
// ****************************************************************************
∀ particles void ini_αβγ(void)
  out (particle α,β,γ) @ -5.0 if (option_particle_frame) {
  if (uid==0) info()<<"[7;37m[ini_αβγ][m";
  α=1.0;
  β=γ=0.0;
 }


// ****************************************************************************
// * Compute loop @ ]+0,+∞[
// ****************************************************************************
void dbgLoop(void) @ 1.0 {
  printf("\n[7;36m[Loop] #%3d, time=%f[m", iteration,time);
}

// ****************************************************************************
// * particleFrameSampling, aucun in/out, on remplit l'histogramme directement
// ****************************************************************************
∀ particles void particleFrameSampling(void) @ 2.0 {
  ℾ rejet=true;
  ℝ θS3,sinθS3;
  if (uid==0) info()<<"[7;37m[particleFrameSampling][m";
  do{
    // On utilise la même graine & générateur pour toutes les particules
    // La reproductibilité n'est pas acquise ici
    const ℝ rndΓ = gamma_rand();
    const ℝ rnd01 = drand48();
    const ℝ θ=θS3=√(2.0*option_νδt*rndΓ);
    sinθS3=sin(θ);
    rejet=(θ>π)||(sinθS3<rnd01*θ);
  }while (rejet);
  //info()<<"["<<uid<<"] theta="<<θS3;
  const ℝ cosθS3=cos(θS3);
  // Vecteur tangent τ au pôle de S³, aléatoire
  const ℝ φ = 2.0*π*drand48();
  const ℝ μ = -1.0+2.0*drand48();
  // Projection de S³ dans S²
  const ℝ τx=μ;
  const ℝ τy=√(1.0-μ²)*cos(φ);
  const ℝ³ ΩS2=ℝ³(cosθS3,τx*sinθS3,τy*sinθS3);
  const ℝ absΩS2=norm(ΩS2);
  const ℝ³ ΩS2final=ΩS2/absΩS2;
  //info()<<"[7;37m[particleFrameSampling] ΩS2.x="<<ΩS2final.x<<"[m";
  diagnosticSpectrum(acos(ΩS2final.x));
}

  
// ****************************************************************************
// * changeFrameFromParticleToLab
// ****************************************************************************
∀ particles void changeFrameFromParticleToLab(void)
  in (particle α,β,γ)
  inout (particle αLab,βLab,γLab) @ 4.0 if (!option_particle_frame){
  const ℝ denom = 1.0-αLab²;
  if (uid==0) info()<<"[7;37m[changeFrameFromeParticleToLab][m";
  if (fabs(denom)>=option_ε_frame_change){
    const ℝ sqrtDenom = √(denom);
    const ℝ unsSqrtDenom = 1./sqrtDenom;
    const ℝ αp = α*αLab-γ*sqrtDenom;
    const ℝ βp = α*βLab+(αLab*βLab*γ-γLab*β)*unsSqrtDenom;
    const ℝ γp = α*γLab+(αLab*βLab*γ+βLab*β)*unsSqrtDenom;
    αLab = αp;
    βLab = βp;
    γLab = γp;
  }else{
    αLab = α*αLab;
    // A revoir !!
    βLab = γ;
    γLab = β;
  }
}


// ****************************************************************************
// * Test for quit
// ****************************************************************************
void tstForQuit(void) @ 4.0 {
  info()<<"[7;37m[tstForQuit][m";
  //printf("\n\t[testForQuit] Iteration=%d, time=%f, delta_t=%f", iteration, time,δt);
  //if (time<option_stoptime) return;
  //printf("\n[7m[tstForQuit] Quit![m");
  dumpSpectrum();
  exit;
}

